package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.bo.Cbb;
import com.brilliance.isc.bo.lg.GetsetStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.GetsetStoreVo;
import com.brilliance.isc.vo.model.Gcacbs;
import org.mapstruct.*;
import org.mapstruct.factory.Mappers;


/**
 * Getset Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface GetsetStructMapper extends IBaseStructMapper<GetsetStoreBo, GetsetStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    GetsetStructMapper INSTANCE = Mappers.getMapper(GetsetStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    GetsetStoreBo voTransferToBo(GetsetStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({
            @Mapping(target = "gidgrp",ignore = true),
            @Mapping(target = "gcdgrp",ignore = true)
    })
    void voUpdateToBo(GetsetStoreVo vo, @MappingTarget GetsetStoreBo bo);

    @AfterMapping
    default void afterUpdate(GetsetStoreVo vo, @MappingTarget GetsetStoreBo bo) {
        Gcacbs cbs = vo.getGcdgrp().getCbs();
        if(cbs==null){
            return;
        }
        Cbb opn1 = cbs.getOpn1();
        bo.getGcdgrp().getCbs().setOpn1(opn1);
        bo.getGcdgrp().getRec().setCannowflg(vo.getGcdgrp().getRec().getCannowflg());
        bo.getGcdgrp().getRec().setFrepayflg(vo.getGcdgrp().getRec().getFrepayflg());
    }
    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    GetsetStoreVo boTransferToVo(GetsetStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(GetsetStoreBo bo, @MappingTarget GetsetStoreVo vo);
}
