package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.bo.lg.GitcomStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.IGctMapper;
import com.brilliance.isc.transfer.model.IGitMapper;
import com.brilliance.isc.transfer.model.IOtherModMapper;
import com.brilliance.isc.vo.lg.GitcomStoreVo;
import org.mapstruct.*;
import org.mapstruct.factory.Mappers;


/**
 * Gitame Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        IGitMapper.class, IGctMapper.class, IOtherModMapper.class}
        )
public interface GitcomStructMapper extends IBaseStructMapper<GitcomStoreBo, GitcomStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    GitcomStructMapper INSTANCE = Mappers.getMapper(GitcomStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    GitcomStoreBo voTransferToBo(GitcomStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({
            @Mapping(target = "oldgidgrp",ignore = true),
            @Mapping(target = "gidgrp",ignore = true)
    })
    void voUpdateToBo(GitcomStoreVo vo, @MappingTarget GitcomStoreBo bo);

    @AfterMapping
    default void afterUpdate(GitcomStoreVo vo, @MappingTarget GitcomStoreBo bo) {
        bo.getGidgrp().getRec().setFeecoldat(vo.getGidgrp().getRec().getFeecoldat());
    }

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    GitcomStoreVo boTransferToVo(GitcomStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(GitcomStoreBo bo, @MappingTarget GitcomStoreVo vo);
}
