package com.brilliance.isc.transfer.transaction;

import com.brilliance.mda.runtime.mda.util.Systems;
import com.brilliance.isc.bo.lg.HitopnStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.HitopnStoreVo;
import com.brilliance.isc.vo.model.CbbVo;
import com.brilliance.isc.vo.model.Giacbs;
import org.mapstruct.AfterMapping;
import org.mapstruct.Mapper;
import org.mapstruct.MappingTarget;
import org.mapstruct.Mappings;
import org.mapstruct.factory.Mappers;


/**
 * Gitame Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface HitopnStructMapper extends IBaseStructMapper<HitopnStoreBo, HitopnStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    HitopnStructMapper INSTANCE = Mappers.getMapper(HitopnStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    HitopnStoreBo voTransferToBo(HitopnStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({
//            @Mapping(target = "oldgidgrp",ignore = true)
    })
    void voUpdateToBo(HitopnStoreVo vo, @MappingTarget HitopnStoreBo bo);

    @AfterMapping
    default void afterUpdate(HitopnStoreVo vo, @MappingTarget HitopnStoreBo bo){
        Giacbs cbs = vo.getNidgrp().getCbs();
        if(cbs == null) return;
        CbbVo max = cbs.getMax();
        if(max == null) return;
        bo.getNidgrp().setAmtwrdtxtz(Systems.moneycnsmalltobig(String.valueOf(max.getAmt())));
        bo.getNidgrp().setFmtamtz(String.valueOf(max.getAmt()));
    }

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    HitopnStoreVo boTransferToVo(HitopnStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(HitopnStoreBo bo, @MappingTarget HitopnStoreVo vo);
}
