package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.bo.lg.NitcomStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.NitcomStoreVo;
import org.mapstruct.*;
import org.mapstruct.factory.Mappers;


/**
 * Gitame Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface NitcomStructMapper extends IBaseStructMapper<NitcomStoreBo, NitcomStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    NitcomStructMapper INSTANCE = Mappers.getMapper(NitcomStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    NitcomStoreBo voTransferToBo(NitcomStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({
            @Mapping(target = "oldnidgrp",ignore = true),
            @Mapping(target = "nidgrp",ignore = true)
    })
    void voUpdateToBo(NitcomStoreVo vo, @MappingTarget NitcomStoreBo bo);

    @AfterMapping
    default void afterUpdate(NitcomStoreVo vo, @MappingTarget NitcomStoreBo bo) {
        bo.getNidgrp().getRec().setFeecoldat(vo.getNidgrp().getRec().getFeecoldat());
    }

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    NitcomStoreVo boTransferToVo(NitcomStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(NitcomStoreBo bo, @MappingTarget NitcomStoreVo vo);
}
